import { DOMRenderer, DataObject, VNode, JsxChildren, JsxProps } from "dom-renderer";
import { CustomElement, DelegateEventHandler, CustomFormElement } from "web-utility";
import { IReactionPublic } from "mobx";
export interface ComponentMeta extends ElementDefinitionOptions, Partial<ShadowRootInit> {
    tagName: string;
}
export type ClassComponent = CustomElementConstructor;
export interface WebCell extends CustomElement {
    internals: ElementInternals;
    renderer: DOMRenderer;
    root: ParentNode;
    update: () => void;
    emit: (event: string, detail?: any, option?: EventInit) => boolean;
}
interface DelegatedEvent {
    type: keyof HTMLElementEventMap;
    selector: string;
    handler: EventListener;
}
/**
 * `class` decorator of Web components
 */
export function component(meta: ComponentMeta): <T extends CustomElementConstructor>(Class: T, { addInitializer }: ClassDecoratorContext<ClassComponent>) => T;
/**
 * Method decorator of DOM Event delegation
 */
export function on<T extends HTMLElement>(type: DelegatedEvent['type'], selector: string): (method: DelegateEventHandler, { addInitializer }: ClassMethodDecoratorContext<T>) => void;
export function getMobxData<T extends DataObject>(observable: T): T;
export type PropsWithChildren<P extends DataObject = {}> = P & {
    children?: JsxChildren;
};
export type FunctionComponent<P extends DataObject = {}> = (props: P) => VNode;
export type FC<P extends DataObject = {}> = FunctionComponent<P>;
export type WebCellComponent = FunctionComponent | ClassComponent;
/**
 * `class` decorator of Web components for MobX
 */
export function observer<T extends WebCellComponent>(func: T, _: ClassDecoratorContext): T;
export function observer<T extends WebCellComponent>(func: T): T;
/**
 * `accessor` decorator of MobX `@observable` for HTML attributes
 */
export function attribute<C extends HTMLElement, V>(_: ClassAccessorDecoratorTarget<C, V>, { name, addInitializer }: ClassAccessorDecoratorContext<C>): void;
export type ReactionExpression<I = any, O = any> = (data?: I, reaction?: IReactionPublic) => O;
export type ReactionEffect<V> = (newValue: V, oldValue: V, reaction: IReactionPublic) => any;
/**
 * Method decorator of MobX `reaction()`
 */
export function reaction<C extends HTMLElement, V>(expression: ReactionExpression<C, V>): (effect: ReactionEffect<V>, { addInitializer }: ClassMethodDecoratorContext<C>) => void;
export interface WebField extends CustomFormElement, WebCell {
}
/**
 * `class` decorator of Form associated Web components
 */
export function formField<T extends ClassComponent>(Class: T, _: ClassDecoratorContext): T;
export type ComponentTag = string | WebCellComponent;
export type WebCellProps<T extends HTMLElement = HTMLElement> = JsxProps<T>;
export interface AsyncBoxProps extends WebCellProps {
    loader: () => Promise<ComponentTag>;
    delegatedProps?: WebCellProps;
}
export interface AsyncBox extends WebCell {
}
export class AsyncBox extends HTMLElement {
    props: AsyncBoxProps;
    loader: AsyncBoxProps['loader'];
    accessor component: FC<PropsWithChildren>;
    accessor delegatedProps: AsyncBoxProps['delegatedProps'];
    connectedCallback(): void;
    protected load(): Promise<void>;
    render(): import("dom-renderer").VNode;
}
type GetAsyncProps<T> = T extends () => Promise<{
    default: FunctionComponent<infer P> | ClassComponent;
}> ? P : {};
export function lazy<T extends () => Promise<{
    default: FunctionComponent | ClassComponent;
}>>(loader: T): (props: GetAsyncProps<T>) => import("dom-renderer").VNode;

//# sourceMappingURL=index.d.ts.map
